local __FILE__=tostring(debugstack(1,2,0):match("(.*):1:")) -- MUST BE LINE 1
local MAJOR_VERSION = ("LibArrow-1.0.lua"):gsub(".lua","")
local MINOR_VERSION = tonumber(string.sub("$Revision: 26 $", 12, -3))
--- **LibArrow-1.0** 
-- \\
-- This library (when used embedded) needs some libraries.\\
-- I didnt hard include them inside the embeddable folder (LibArrow-1.0) so
-- you should add these dependencies yourself
--
-- LibArrow-1.0 uses: \\
--	**LibStub** \\
--	**CallbackHandler-1.0** \\
--	**Astrolabe** \\
-- \\
-- or in your pkgmeta \\
-- externals: \\
--	Libs/CallbackHandler-1.0: svn://svn.wowace.com/wow/callbackhandler/mainline/trunk/CallbackHandler-1.0 \\
--  Libs/LibStub: svn://svn.wowace.com/wow/libstub/mainline/trunk \\
--  Libs/Astrolabe: http://svn.esamynn.org/astrolabe/trunk@124 \\
-- Check release number for astrolabe in your files. The number you see here is the one I tested the library against and it will change when needed\\
-- Astrolabe author advice to not link directly to trunk, but you can do it if you feel brave enough: \\
-- Astrolabe's source code repository can be found here: http://svn.esamynn.org/astrolabe/
-- The trunk/ is generally stable, and fit for regular use. Development that may destabilize stuff is generally done in a branch.
-- I generally DO NOT recommend using Subversion externals to include Astrolabe's code in your own project. However, if you insist on doing so, please be sure to specify a specific revision in your externals property.
-- @usage
-- local arrow=LibStub("LibArrow-1.0"):NewArrow()
-- arrow:SetLocalWaypoint(0.5,0.5)
-- @class file
-- @name LibArrow-1.0

--- LibArrow-1.0 raises 3 messages
-- * FAR When distance goes over 10 and the arrow change from the "you are here" model to the "you are far" model
-- * NEAR When distance goes under 10 and the arrow change from the "you are far" model to the "you are near" model
-- * MOVED When the arrow is repositioned. Get called when the arrow is in the NEW position. Use it to save position information
-- Event handling is done via CallbackHandler-1.0
-- @usage
-- local LA=LibStub("LibArrow-1.0")
-- LA.RegisterCallback(myaddonobject,"RESIZED","SaveArrowPosition")
-- @name Messages
local me, ns = ...
local _,_,_,toc=GetBuildInfo()
local pp=print
--[===[@debug@
print("Loading",__FILE__," inside ",me)
--@end-debug@]===]
--[[
Name: LibBattleScan.lua
Revision: $Rev: 26 $
Author: Alar of Daggerspine
Email: alar@aspide.it
Website: http://www.curse.com
SVN: $HeadUrl:$
License: GPL v2.1
--]]
local lib,old=LibStub:NewLibrary(MAJOR_VERSION,MINOR_VERSION)
if (not lib) then
	--[===[@debug@
	print("Already loaded a newer or equal version of " .. MAJOR_VERSION)
	--@end-debug@]===]
	return -- Already loaded
end
if (old) then
	--[===[@debug@
	print(format("Upgrading %s from %s to %s",MAJOR_VERSION,old,MINOR_VERSION))
	--@end-debug@]===]
end
lib.pois=lib.pois or {}
lib.arrows=lib.arrows or {}
lib.events=lib.events or LibStub("CallbackHandler-1.0"):New(lib)
if (LibDebug) then LibDebug() end
local function debug(...) 
	--[===[@debug@
	print(...)
	--@end-debug@]===]
end
local print=_G.print
local notify=_G.print
local error=_G.error
local function dump() end
local function debugEnable() end
-- Dont worry about this... if i have around some utility 
-- functions from my other addos I use then, otherwise just stick to standard
if (LibStub("AlarLoader-3.0",true)) then
	local rc=LibStub("AlarLoader-3.0"):GetPrintFunctions(MAJOR_VERSION)
	print=rc.print
	--[===[@debug@
	debug=rc.debug
	dump=rc.dump
	--@end-debug@]===]
	notify=rc.notify
	error=rc.error
	debugEnable=rc.debugEnable
else
	debug("Missing AlarLoader-3.0")
end
debugEnable(true)
--[[ Standard prologue end --]]
-- Dynamically retrieving texture path
local mx=me:gsub("-",".")
local cp = __FILE__:match(mx.."(.*)")
local path="Interface\\AddOns\\" .. me .. cp
path=path:gsub("LibArrow.1.0.lua","")
local arrowUp=path .."arrow_image"
local arrowDown=path .."arrow_image_down"
local Astrolabe
Astrolabe=DongleStub("Astrolabe-1.0")

local arrows=lib.arrows
local pois=lib.pois
--[[ This entire file is pretty liberally ganked from TomTom (and then modified) under the following license:

-------------------------------------------------------------------------
  Copyright (c) 2006-2007, James N. Whitehead II
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials provided
        with the distribution.
      * The name or alias of the copyright holder may not be used to endorse 
        or promote products derived from this software without specific prior
        written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
---------------------------------------------------------------------------

Thanks, James! <3   ]]

local function ColorGradient(perc, ...)
	local num = select("#", ...)
	local hexes = type(select(1, ...)) == "string"

	if perc == 1 then
		return select(num-2, ...), select(num-1, ...), select(num, ...)
	end

	num = num / 3

	local segment, relperc = math.modf(perc*(num-1))
	local r1, g1, b1, r2, g2, b2
	r1, g1, b1 = select((segment*3)+1, ...), select((segment*3)+2, ...), select((segment*3)+3, ...)
	r2, g2, b2 = select((segment*3)+4, ...), select((segment*3)+5, ...), select((segment*3)+6, ...)

	if not r2 or not g2 or not b2 then
		return r1, g1, b1
	else
		return r1 + (r2-r1)*relperc,
		g1 + (g2-g1)*relperc,
		b1 + (b2-b1)*relperc
	end
end



local function OnEvent(self,event,...)
end
local function OnDragStart(self, button)
--[===[@debug@
	print("DragStart")
--@end-debug@]===]
	if not self.obj:Locked() then 
		self:StartMoving()
	end
end

local function OnDragStop(self, button)
--[===[@debug@
print("DragStop")
--@end-debug@]===]
	self:StopMovingOrSizing()
	local point,parent,relpoint,x,y=self:GetPoint()
	if (parent) then parent=parent:GetName() end
	self.obj:Fire("MOVED",point,parent,relpoint,x,y)
end
local function GetHeading(m,f,x,y)
	if not (m and f and x and y) then return end
	local d,dx,dy
	if (Astrolabe) then
		local m1,f1,x1,y1=Astrolabe:GetCurrentPlayerPosition()
		d,dx,dy=Astrolabe:ComputeDistance(m1,f1,x1,y1,m,f,x,y)
	else
		local px,py=GetPlayerMapPosition("player")
		dx=x-px
		dy=y-py
		d = sqrt(dx*dx + dy*dy) * 100
	end
	if not (dx and dy) then return end
	local a = atan2(-dx, -dy) / 360 * (math.pi * 2) -- degrees. seriously what
	a = a - GetPlayerFacing()
    if (not tonumber(d)) then d = 9999 end
    return a,d
end

local function OnUpdate(self,elapsed,...)
	self.e=self.e +elapsed
	if self.e < self.interval then return end
	self.e=0
	local showDownArrow=self.showDownArrow
	local arrow=self.arrow
	local title=self.title
	local status=self.status
    local o=self.obj
    local a,d=GetHeading(o.m,o.f,o.x,o.y)
    if (not a) then return end
    status:SetText(format("%d",d)) -- TODO: improve the display :)
	-- Showing the arrival arrow?
	if d and d <= 10 then
		if not showDownArrow then
			o:Fire("NEAR")
			showDownArrow = true
			arrow:SetTexture(arrowDown)
			if (o.forcecolor) then
				arrow:SetVertexColor(o.r,o.g,o.b)
			else
				arrow:SetVertexColor(0, 1, 0)
			end
		end
		local cell = math.floor(mod(GetTime() * 20, 55)) -- 20 fps seems to be around the right number
		local column = cell % 9
		local row = floor(cell / 9)
		local xstart = (column * 53) / 512
		local ystart = (row * 70) / 512
		local xend = ((column + 1) * 53) / 512
		local yend = ((row + 1) * 70) / 512
		arrow:SetTexCoord(xstart,xend,ystart,yend)
	else
		if showDownArrow then
			o:Fire("FAR")
			arrow:SetTexture(arrowUp)
			showDownArrow = false
			if (o.forcecolor) then
				arrow:SetVertexColor(o.r,o.g,o.b)
			end
		end
		if (not o.forcecolor) then
			local perc = math.abs((math.pi - math.abs(a)) / math.pi)
	    	if perc > 1 then perc = 2 - perc end -- siiigh
	    	if perc <0 then perc=0.1 end
			local gr,gg,gb = 0, 1, 0
			local mr,mg,mb = 1, 1, 0
			local br,bg,bb = 1, 0, 0
			local s,r,g,b = pcall(ColorGradient,perc, br, bg, bb, mr, mg, mb, gr, gg, gb)
			if (s) then		
				arrow:SetVertexColor(r,g,b)
			end
		end
		local cell = floor(a / (math.pi * 2) * 108 + 0.5) % 108
		local column = cell % 9
		local row = floor(cell / 9)

		local xstart = (column * 56) / 512
		local ystart = (row * 42) / 512
		local xend = ((column + 1) * 56) / 512
		local yend = ((row + 1) * 42) / 512
		arrow:SetTexCoord(xstart,xend,ystart,yend)
	end
	self:Show()
	self.showDownArrow=showDownArrow
end

local registry=lib.events
local arrow={}

function arrow:Fire(event,...)
	registry:Fire(event,self,...)
end
function arrow:Show()
	self.frame:Show()
end
function arrow:Hide()
	self.frame:Hide()
end
function arrow:SetPoint(...)
	self.frame:ClearAllPoints() -- We only want one point for this arrow
	self.frame:SetPoint(...)
end
function arrow:GetPoint()
	self.frame:GetPoint(1)
end
function arrow:SetScale(...)
	self.frame:SetScale(...)
end
function arrow:SetSize(size)
	self.frame:SetHeight(size)
	self.frame:SetWidth(size)
end
function arrow:GetSize(size)
	self.frame:GetHeight()
end
function arrow:ClearAllPoints(...)
	self.frame:ClearAllPoints(...)
end
function arrow:Locked()
	return self.frame.locked
end
--- Makes arrow unmovable
function arrow:Lock()
	self.frame.locked=true
end
--- Makes arrow movable
function arrow:Unlock()
	self.frame.locked=false
end
function arrow:Test()
	self:SetWaypoint(Astrolabe:GetCurrentPlayerPosition())
end
function arrow:SetColor(r,g,b,a)
	if (r) then
		self.forcecolor=true
		self.r=r
		self.g=g or 0
		self.b=b or 0
		self.a=a or 1
		self.frame.arrow:SetVertexColor(self.r,self.g,self.b,self.a)		
	else
		self.forcecolor=false
	end
	OnUpdate(self.frame,self.frame.interval+1)
end
--- Set the frequency for arrow updating. Values lower than 20 are not nice
function arrow:SetFrequency(frequency)
	self.frame.interval=1/frequency
end
--- Set a waypoint in the current map
-- @paramsig x,y
function arrow:SetLocalWaypoint(x,y)
	local m,f=Astrolabe:GetCurrentPlayerPosition()
	self:SetWaypoint(m,f,x,y)
end
--- Set a waypoint anywhere in the world
-- Usually you obtain this values from astrolabe
-- @param m  areaID or zone name
-- @param f  floor or nil
-- @param x  x coord
-- @param y  y coord

function arrow:SetWaypoint(m,f,x,y)
	if (type(m)=="string") then -- Old Style coordinate
		m=Astrolabe:GetMapID(m,f)
		f=0
	end
	self.m=m
	self.f=f or 0
	self.x=x
	self.y=y
	OnUpdate(self.frame,1000)
end
--- Set a custom value. Dont use the main object to store your data

function arrow:SetUserData(key,userdata)
	self.userdata[key]=userdata
end
--- Retrieve a value set with SetUserData
function arrow:GetUserData(key)
	return self.userdata[key]
end
--- Remove minimap icon for current waypoint
function arrow:RemoveIcon()
	if self.icon then
		self.icon.frame:Hide()
		tinsert(pois,self.icon.frame) 
		Astrolabe:RemoveIconFromMinimap(self.icon.frame) 
	end
end
--- Adds  a minimap icon for the current waypoint
-- @paramsig same as setTexture()
function arrow:AddIcon(...)
	self:RemoveIcon()
	self.icon=lib:NewIcon(...)
	self.icon.frame:Show()
	Astrolabe:PlaceIconOnMinimap(self.icon.frame,self.m,self.f,self.x,self.y)
end
--- Set text for the status message
-- @param string text 
function arrow:SetTitleText(text)
	self.frame.title:SetText(text)
end
--- Set color for the title message
-- @paramsig r,g,b[,a]
function arrow:SetTitleColor(...)
	self.frame.title:SetTextColor(...)
end
--- Set text for the status message
-- @param string text 
function arrow:SetStatusText(text)
	self.frame.status:SetText(text)
end
--- Set color for the status message
-- @paramsig r,g,b[,a]
function arrow:SetStatusColor(...)
	self.frame.status:SetTextColor(...)
end
function arrow:Release()
	self:Reset()
	tinsert(arrows,self.frame)
	self.frame=nil
	setmetatable(self,nil)
end
function arrow:Reset()
	self:SetWaypoint()
	self:ClearAllPoints()
	self:RemoveIcon()
	self.frame:Hide()
	self.frame:SetScript("OnUpdate",nil)
	wipe(self.userdata)
	self.e=0
end
local counter=0
--- Creates a new arrow object
--@paramsig use the same parameter as frame:SetPoint(). default is setPoint("CENTER") 
--@params 
function lib:NewArrow(...)
	local wayframe =tremove(arrows)
	if (not wayframe) then	
		counter=counter+1
		wayframe =CreateFrame("Button", "LibArrowFrame" ..counter, UIParent)
		local titleframe = CreateFrame("Frame", nil, wayframe)
		wayframe.title = titleframe:CreateFontString("OVERLAY", nil, "GameFontNormalSmall")
		wayframe.status = titleframe:CreateFontString("OVERLAY", nil, "GameFontNormalSmall")
		wayframe.tta = titleframe:CreateFontString("OVERLAY", nil, "GameFontNormalSmall")
		wayframe.title:SetPoint("TOP", wayframe, "BOTTOM", 0, 0)
		wayframe.status:SetPoint("TOP", wayframe.title, "BOTTOM", 0, 0)
		wayframe.tta:SetPoint("TOP", wayframe.status, "BOTTOM", 0, 0)
		wayframe:RegisterForDrag("LeftButton")
		wayframe.arrow = wayframe:CreateTexture("OVERLAY")
		wayframe.arrow:SetTexture(arrowUp)	
		wayframe.arrow:SetAllPoints(wayframe)
	end
	wayframe:Hide()
	wayframe.title:SetText(wayframe:GetName())
	wayframe:SetHeight(50)
	wayframe:SetWidth(50)
	wayframe:SetScale(1)
	wayframe.interval=0.05
	wayframe.title:SetTextColor(1,0.8,0)
	wayframe.status:SetTextColor(1,0.8,0)
	wayframe.e=0
	if (select('#',...)==0) then
		wayframe:SetPoint("CENTER", 0,0)
	else
		wayframe:SetPoint(...)
	end
	wayframe:EnableMouse(true)
	wayframe:SetMovable(true)
	-- Frame used to control the scaling of the title and friends
	wayframe:SetScript("OnDragStart", OnDragStart)
	wayframe:SetScript("OnDragStop", OnDragStop)
	wayframe:SetScript("OnUpdate",OnUpdate)
	wayframe:SetScript("OnEvent",OnEvent)
	local widget=setmetatable({frame=wayframe,userdata={}},{__index=arrow})
	wayframe.obj=widget
	return widget
end
local poi={}
function poi:Change(texture,...)
	if (texture) then
		self.frame.arrow.texture:SetTexture(texture)
	end
	if (select('#',...)>0) then
		self.frame.arrow.texture:SetTexCoord(...)
	end
end
function lib:NewIcon(texture,...)
	local icon=tremove(pois)
	if (not icon) then
		counter=counter+1
		icon=CreateFrame("Button", "LibArrowIcon" ..counter, Minimap)
		icon:SetHeight(10)
		icon:SetWidth(10)
		icon:EnableMouse()
		local arrow = CreateFrame("Frame", nil, icon)
		arrow:SetPoint("CENTER", icon)
		arrow:SetWidth(16)
		arrow:SetHeight(16)
		local arrowtexture = arrow:CreateTexture(nil, "OVERLAY")
		arrowtexture:SetAllPoints(arrow)
		arrow.texture = arrowtexture
		arrow.t = 0
		arrow.icon = icon
		icon.arrow = arrow
		icon:Show()
	end
	icon.arrow.texture:SetTexture(texture)
	if (select('#',...)>0) then
		icon.arrow.texture:SetTexCoord(...)
	end
	local widget=setmetatable({frame=icon},{__index=poi})
	icon.obj=widget
	return widget
end
function lib:LoadedFrom()
	return __FILE__
end
	
